import csv
from my_collections.petroleum_data import PetroleumData, AggregatedPetroleumData

'''
"chokes": percentage choke openings
"_export": export measurements
"_multi": multiphase measurements
N = number of chokes, M = number of points
Size "chokes", "_multi" = M x N
Size "_export" = M x 1
'''


def read_raw_data():
    num_wells = 11
    cols_per_choke = 5
    col_start = 1

    with open('data_files/input/gjoa.csv', 'r') as csv_file:
        data_reader = csv.reader(csv_file, delimiter=";")

        # Skip header of file
        for _ in range(2):
            next(data_reader)

        # Read data file
        chokes, oil_multi, gas_multi, oil_export, gas_export = [], [], [], [], []
        for row in data_reader:
            gas_multi.append([])
            oil_multi.append([])
            chokes.append([])
            for i in range(num_wells):
                j = col_start + i*cols_per_choke
                gas_multi[-1].append(float(row[j]))
                oil_multi[-1].append(float(row[j+1]))
                chokes[-1].append(float(row[j+4])/100)
            gas_export.append(float(row[col_start + num_wells*cols_per_choke+1]))
            oil_export.append(float(row[col_start + num_wells*cols_per_choke]))

    # Returns one data-set for oil and one data-set for gas
    return PetroleumData(chokes, oil_export, oil_multi, "oil"), PetroleumData(chokes, gas_export, gas_multi, "gas")


def read_aggregated_data():
    num_wells = 11
    col_start = 3

    with open('data_files/input/gjoa_aggregated.csv', 'r') as csv_file:
        data_reader = csv.reader(csv_file, delimiter=";")

        # Skip header of file
        for _ in range(2):
            next(data_reader)

        chokes, oil, gas, chokes_var, oil_var, gas_var = [], [], [], [], [], []

        for row in data_reader:
            chokes.append([])
            chokes_var.append([])
            # Choke values are scaled to the range [0, 1]
            for i in range(num_wells):
                chokes[-1].append(float(row[col_start + 2*i])/100)
                chokes_var[-1].append(float(row[col_start + 2*i + 1])/100**2)
            oil.append(float(row[col_start + 2*num_wells]))
            oil_var.append(float(row[col_start + 2*num_wells + 1]))
            gas.append(float(row[col_start + 2*num_wells + 2]))
            gas_var.append(float(row[col_start + 2*num_wells + 3]))

    return AggregatedPetroleumData(chokes, oil, chokes_var, oil_var, "oil"), AggregatedPetroleumData(chokes, gas, chokes_var, gas_var, "gas")
