import csv
import numpy as np
import openpyxl
import os.path
from my_collections.petroleum_data import include_time
from misc import normalize_moments


def read_data_models(name):
    with open('data_files/output/{}_models.csv'.format(name), 'r') as csv_file:
        data_reader = csv.reader(csv_file, delimiter=";")
        num_variables = int(next(data_reader)[0])
        moments, corr = [], []
        for _ in range(4):
            row = next(data_reader)
            moments.append([float(i) for i in row])
        for _ in range(num_variables):
            row = next(data_reader)
            corr.append([float(i) for i in row])
        return np.array(moments), np.array(corr)


def save_data_models(moments, corr, name):
    with open('data_files/output/{}_models.csv'.format(name), 'w') as csv_file:
        data_writer = csv.writer(csv_file, delimiter=";", lineterminator='\n')
        data_writer.writerow([len(moments[0])])
        for row in moments:
            data_writer.writerow(row)
        for row in corr:
            data_writer.writerow(row)


def save_coefficients(regression_coefficients, name):
    regression_coefficients = np.array(regression_coefficients)
    with open('data_files/output/{}_coeff.csv'.format(name), 'w') as csv_file:
        data_writer = csv.writer(csv_file, delimiter=";", lineterminator='\n')
        for i in range(regression_coefficients.shape[1]):
            data_writer.writerow(regression_coefficients[:, i])


def read_coefficients(name):
    with open('data_files/output/{}_coeff.csv'.format(name), 'r') as csv_file:
        data_reader = csv.reader(csv_file, delimiter=";", lineterminator='\n')
        regression_coefficients = []
        for row in data_reader:
            regression_coefficients.append([float(i) for i in row])
    return np.array(regression_coefficients).T


def read_modified_moments(name):
    sheet = openpyxl.load_workbook("data_files/output/modified_moments.xlsx", data_only=True).get_sheet_by_name(name)
    moments = []
    for col in "BCDF":
        moments.append([float(sheet[col+str(row)].value) for row in range(2, sheet.max_row+1)])
    return np.array(moments)


def save_modified_moments(moments, num_wells, name):
    filename = "data_files/output/modified_moments.xlsx"
    norm_moments = normalize_moments(moments)
    book = openpyxl.Workbook() if not os.path.isfile(filename) else openpyxl.load_workbook(filename)
    num_var = moments.shape[1]
    if name not in book.get_sheet_names():
        sheet = book.create_sheet(title=name)
        sheet.column_dimensions['A'].width = 20
        for c in "BCDEFG":
            sheet.column_dimensions[c].width = 15
        for heading, col in zip(["Mean", "Var", "Skew", "Skew (norm)", "Kurt", "Kurt (norm)"], "BCDEFG"):
            sheet[col+'1'] = heading
        for i in range(num_var):
            if i == num_var - 1 and include_time:
                title = "Time"
            elif i > num_var - num_wells - (2 if include_time else 1):
                title = "Slope well {}".format(i - (num_var - num_wells) + (2 if include_time else 1))
            else:
                title = "Constant well {}".format(i+1)
            row = str(i+2)
            sheet['A'+row] = title
            sheet['D'+row] = "=E{0}*C{0}^(1.5)".format(row)
            sheet['F'+row] = "=G{0}*C{0}^2".format(row)
    else:
        sheet = book.get_sheet_by_name(name)
    for i in range(num_var):
        row = str(i+2)
        sheet['B'+row] = moments[0, i]
        sheet['C'+row] = moments[1, i]
        sheet['E'+row] = norm_moments[2, i]
        sheet['G'+row] = norm_moments[3, i]
    if 'Sheet' in book.get_sheet_names():
        book.remove_sheet(book.get_sheet_by_name('Sheet'))
    book.save(filename)


