from input_output import data_io, model_io, mosel_io
import data_modelling.bootstrap as dm
import statsmodels.api as sm
import numpy as np
import tests
from misc import plot_histogram


# Parameters for this script
use_export = True
use_multiphase = True
use_saved_data = True
compress_data = True
num_wells = 11
gas_wells = [1, 8, 9, 10]


def plot_bootstrap_distributions():
    a = np.empty((4, num_wells*2+1))
    oil_coeff = remove_gas_wells_and_time(a, coeff=model_io.read_coefficients("oil"))
    gas_coeff = remove_gas_wells_and_time(a, coeff=model_io.read_coefficients("gas"))
    plot_histogram(oil_coeff, oil_moments, 7, _include_time=False, save_as="oil")
    plot_histogram(gas_coeff, gas_moments, 7, _include_time=False, save_as="gas")


# Removes variables from moments/corr that should not be used in the scenario generation
def remove_gas_wells_and_time(moments, corr=None, coeff=None):
    if moments.shape[1] == num_wells*2 + 1:
        # Gas well constants + gas well contributions + time variable
        indices = gas_wells + [num_wells + i for i in gas_wells] + [2*num_wells]
    else:
        # Gas well contributions + time variable
        indices = np.array(gas_wells + [num_wells]) + 1
    if coeff is not None:
        return np.delete(coeff, indices, axis=0)
    moments = np.delete(moments, indices, axis=1)
    if corr is not None:
        corr = np.delete(corr, indices, axis=0)
        corr = np.delete(corr, indices, axis=1)
        return moments, corr
    return moments


oil_data, gas_data = data_io.read_raw_data()

if use_saved_data:
    oil_moments, oil_corr = model_io.read_data_models("oil")
    gas_moments, gas_corr = model_io.read_data_models("gas")
else:
    if compress_data:
        oil_data.compress()
        gas_data.compress()
    oil_moments, oil_corr = dm.residuals_resampling(oil_data, use_export, use_multiphase, sm.robust.norms.HuberT)
    gas_moments, gas_corr = dm.residuals_resampling(gas_data, use_export, use_multiphase, sm.robust.norms.HuberT)
    modified_oil_moments, modified_gas_moments = oil_moments, gas_moments
    model_io.save_data_models(oil_moments, oil_corr, "oil")
    model_io.save_data_models(gas_moments, gas_corr, "gas")


oil_moments, oil_corr = remove_gas_wells_and_time(oil_moments, oil_corr)
gas_moments, gas_corr = remove_gas_wells_and_time(gas_moments, gas_corr)
chokes = oil_data.chokes[:, [i for i in range(num_wells) if i not in gas_wells]]

mosel_ig = mosel_io.MoselInputGenerator(chokes, gas_data.export, oil_data.export, oil_moments, oil_corr,
                                        gas_moments, gas_corr)

# mosel_ig.run(num_points=4, num_scenarios=100, num_test_scenarios=1000, delta_gas=-50000, penalty=10)
# tests.test_synthetic_case1()
# tests.test_synthetic_case2()
# tests.test_synthetic_case3()
# tests.test_synthetic_case4()
# tests.test_real_case(mosel_ig)

