import numpy as np
import scipy.optimize
import warnings
from misc import compute_moments

warnings.filterwarnings('ignore', 'The iteration is not making good progress')


mom_memo = {}


def mom(a, i):
    if i not in mom_memo:
        mom_memo[i] = np.mean(a**i)
    return mom_memo[i]


def mom_func(p, _x, n):
    c = p
    for i in range(n-1):
        c = np.convolve(p, c)
    return sum(c[i] * mom(_x, i) for i in range(4+3*(n-1)))


def equations(p, _x, i, target_mom):
    global mom_memo
    mom_memo = {}
    return [(mom_func(p, _x[i], j+1) - target_mom[j][i])**2 for j in range(target_mom.shape[0])]


def _distance_mom(m1, m2):
    return np.mean((m1 - m2)**2)**0.5


def transform(_x, target_mom):
    num_mom = target_mom.shape[0]
    epsilon_x = 0.2 if _x.shape[1] > 5 else 0.1
    max_iter = 3
    error1 = _distance_mom(compute_moments(_x, num_mom), target_mom)
    j = 0
    while error1 > epsilon_x and j < max_iter:
        for i in range(len(_x)):
            p = [0., 1.] + [0.]*(num_mom-2)
            p = scipy.optimize.fsolve(equations, p, args=(_x, i, target_mom))
            _x[i] = sum(p[j]*_x[i]**j for j in range(num_mom))
        error2 = _distance_mom(compute_moments(_x, num_mom), target_mom)
        j = j + 1 if error1 - error2 < 1e-4 else 0
        error1 = error2
    return _x